/*--------------------------------------------------------------------------------------------------*/
/* 					Function for the creation of the 3 dimensional Communication grid				*/
/*    																							   	*/
/*                                                                           						*/
/* A. P. Liavas																						*/
/* Georgios Lourakis																				*/ 
/* Georgios Kostoulas																				*/ 
/* 15/11/2017                                             											*/
/*--------------------------------------------------------------------------------------------------*/
#include "ntf_functions.h"
#include <Eigen/Dense>

using namespace Eigen;

//	<--------------------------------------		Create Grid		--------------------------------------->	//
void Create_Grid(MPI_Comm *grid_comm, MPI_Comm *frontal_comm, MPI_Comm *horizontal_comm, MPI_Comm *lateral_comm, 
				MPI_Comm *column_comm, MPI_Comm *row_comm, MPI_Comm *tube_comm, int *my_rank, int *frontal_rank, int *horizontal_rank, int *lateral_rank, 
				int *column_rank, int *row_rank, int *tube_rank, int *coordinates, int p_A, int p_B, int p_C){
	
	int dim_sizes[3], wrap_around[3];					// |
	int free_coords[3];									// | Variables for the Communication Grid
	int reorder = 1;									// |
	
	dim_sizes[1] = p_A;	dim_sizes[2] = p_B;	dim_sizes[0] = p_C;
	wrap_around[0] = wrap_around[1] = wrap_around[2] = 1;
	
	MPI_Cart_create(MPI_COMM_WORLD, 3, dim_sizes, wrap_around, reorder, grid_comm);
	MPI_Comm_rank(*grid_comm, my_rank);	
	MPI_Cart_coords(*grid_comm, *my_rank, 3, coordinates);
	
	free_coords[1] = free_coords[2] = 1; free_coords[0] = 0;
	MPI_Cart_sub(*grid_comm, free_coords, frontal_comm);
	
	free_coords[0] = free_coords[2] = 1; free_coords[1] = 0;
	MPI_Cart_sub(*grid_comm, free_coords, horizontal_comm);
	
	free_coords[0] = free_coords[1] = 1; free_coords[2] = 0;
	MPI_Cart_sub(*grid_comm, free_coords, lateral_comm);
	
	MPI_Comm_rank(*frontal_comm, frontal_rank);
	MPI_Comm_rank(*horizontal_comm, horizontal_rank);
	MPI_Comm_rank(*lateral_comm, lateral_rank);
	
	free_coords[0] = free_coords[2] = 0; free_coords[1] = 1;
	MPI_Cart_sub(*grid_comm, free_coords, column_comm);
	
	free_coords[0] = free_coords[1] = 0; free_coords[2] = 1;
	MPI_Cart_sub(*grid_comm, free_coords, row_comm);
	
	free_coords[1] = free_coords[2] = 0; free_coords[0] = 1;
	MPI_Cart_sub(*grid_comm, free_coords, tube_comm);
	
	MPI_Comm_rank(*column_comm, column_rank);
	MPI_Comm_rank(*row_comm, row_rank);
	MPI_Comm_rank(*tube_comm, tube_rank);	
	
	
}
