/*--------------------------------------------------------------------------------------------------*/
/* 						Function for the computation of the Objective Value		 					*/
/*    									of the NTF problem										   	*/
/*              	(make use of results already computed in the update of Factor C)          		*/
/*                        || X_0 - X || = sqrt(<X_0,X_0> + <X,X> - 2 <X_0,X>)		 				*/
/*                		where:   	<X_0,X_0> =  frob_X,											*/
/*                       			<X,X> =  sum(Z),												*/
/*                       			<X_0,X> = global_sum											*/
/*                                                                           						*/
/* A. P. Liavas																						*/
/* Georgios Lourakis																				*/ 
/* Georgios Kostoulas																				*/ 
/* 15/11/2017                                              											*/
/*--------------------------------------------------------------------------------------------------*/
#include "ntf_functions.h"

using namespace Eigen;

double Get_Objective_Value(const Ref<const MatrixXd> C, const Ref<const MatrixXd> X_C_Kr, const Ref<const MatrixXd> A_T_A,
							const Ref<const MatrixXd> B_T_B, const Ref<const MatrixXd> C_T_C, double frob_X, MPI_Comm grid_comm){
	double local_sum, global_sum;
	int n_K = X_C_Kr.rows();
	int R = X_C_Kr.cols();
	
	MatrixXd Z(R, R);
	Z.noalias() = A_T_A.cwiseProduct(B_T_B.cwiseProduct(C_T_C));					// (A^T*A) .* (B^T*B) .* (C^T*C)
	
	local_sum = (X_C_Kr.cwiseProduct(C)).sum();										// sum(sum((X_C * KhatriRao_BA) .* C))
	
	MPI_Allreduce(&local_sum, &global_sum, 1, MPI_DOUBLE, MPI_SUM, grid_comm);		// Communication through all processes
	
	return sqrt(frob_X + Z.sum() - 2*global_sum );
}
