/*--------------------------------------------------------------------------------------------------*/
/* 							HEADER FILE ntf_functions.h			    								*/
/*    		(Containes declerations of functions used by ntf_eigen_mpi.cpp)						   	*/
/*                                                                           						*/
/* A. P. Liavas																						*/
/* Georgios Kostoulas																				*/
/* Georgios Lourakis																				*/ 
/* 15/11/2017                                              											*/
/*--------------------------------------------------------------------------------------------------*/
#ifndef NTF_FUNCTIONS_H		// if ntf_functions.h hasn't been included yet...
#define NTF_FUNCTIONS_H		// #define this so the compiler knows it has been included

#include <Eigen/Dense>
#include <mpi.h>

using namespace Eigen;

void Compute_Grid_Dimensions(int *p_A, int *p_B, int *p_C, int comm_sz, int flag);

void Compute_SVD(double *L, double *mu, const Ref<const MatrixXd> Z);

void Create_Grid(MPI_Comm *grid_comm, MPI_Comm *frontal_comm, MPI_Comm *horizontal_comm, MPI_Comm *lateral_comm, 
				MPI_Comm *column_comm, MPI_Comm *row_comm, MPI_Comm *tube_comm, int *my_rank, int *frontal_rank, int *horizontal_rank, int *lateral_rank, 
				int *column_rank, int *row_rank, int *tube_rank, int *coordinates, int p_A, int p_B, int p_C);
				
void Dis_Count(int *dis, int *Count, int size, int Dim, int R);

void G_Lambda(double *lambda, double *q, double *L, double mu);

double Get_Objective_Value(const Ref<const MatrixXd> C, const Ref<const MatrixXd> X_C_Kr, const Ref<const MatrixXd> A_T_A,
							const Ref<const MatrixXd> B_T_B, const Ref<const MatrixXd> C_T_C, double frob_X, MPI_Comm grid_comm);

double Get_Objective_Value_Accel(const Ref<const MatrixXd> C, const Ref<const MatrixXd> X_C, const Ref<const MatrixXd> Kr, const Ref<const MatrixXd> A_T_A, 
								const Ref<const MatrixXd> B_T_B, const Ref<const MatrixXd> C_T_C, double frob_X, MPI_Comm grid_comm);

void Khatri_Rao_Product(const Ref<const MatrixXd> U2, const Ref<const MatrixXd> U1, Ref<MatrixXd> Kr);

void Line_Search_Accel(const Ref<const MatrixXd> A_old_N, const Ref<const MatrixXd> B_old_N, const Ref<const MatrixXd> C_old_N, Ref<MatrixXd> A, 
						Ref<MatrixXd> B, Ref<MatrixXd> C, Ref<MatrixXd> A_T_A, Ref<MatrixXd> B_T_B, Ref<MatrixXd> C_T_C, Ref<MatrixXd> KhatriRao_BA,
						const Ref<const MatrixXd> X_C, int *acc_fail, int *acc_coeff, int iter, double f_value, double frob_X, 
						MPI_Comm column_comm, MPI_Comm row_comm, MPI_Comm tube_comm, MPI_Comm grid_comm);

void Normalize(Ref<MatrixXd> A, Ref<MatrixXd> B, Ref<MatrixXd> C, Ref<MatrixXd> A_T_A, Ref<MatrixXd> B_T_B, Ref<MatrixXd> C_T_C);

void Normalize_Init(Ref<MatrixXd> A, Ref<MatrixXd> B, Ref<MatrixXd> C, MPI_Comm row_comm, MPI_Comm tube_comm);

void Nesterov_Matrix_Nnls(Ref<MatrixXd> Z, Ref<MatrixXd> W, Ref<MatrixXd> A_init, double delta_1, double delta_2);

void Read_Data(Ref<MatrixXd> A, Ref<MatrixXd> B, Ref<MatrixXd> C, Ref<MatrixXd> X_A, Ref<MatrixXd> X_B, Ref<MatrixXd> X_C, 
						int I, int J, int K, int n_I, int n_J, int n_K, int skip_I, int skip_J, int skip_K, int R, int comm_sz);

void Read_From_File(int nrows, int ncols, Ref<MatrixXd> Mat, const char *file_name, int skip);

void Read_X_C(int n_I, int n_J, int n_K, int skip_I, int skip_J, int skip_K, int I, int J, Ref<MatrixXd> X_C_one_file_T,
						const char *file_name);
	
void Set_Info(int* R, int* I, int* J, int* K, const char *file_name);

void update_alpha(double alpha, double q, double *new_alpha);
							
void Workerjob(int my_rank, int n_J, int n_K, Ref<MatrixXd> A, const Ref<const MatrixXd> C_T_C, const Ref<const MatrixXd> A_T_A, 
				const Ref<const	MatrixXd> X_A, const Ref<const MatrixXd> Kr, int *Count, int *dis, double delta_1, double delta_2, 
				int factor, MPI_Comm comm);

void Workerjob_C(int my_rank, Ref<MatrixXd> C, Ref<MatrixXd> W_C, const Ref<const MatrixXd> B_T_B, const Ref<const MatrixXd> A_T_A,
  				const Ref<const	MatrixXd> X_C, const Ref<const MatrixXd> Kr, int *Count, int *dis, double delta_1, double delta_2,
  				 MPI_Comm comm);
					
						
#endif 
