/*--------------------------------------------------------------------------------------------------*/
/* 							Function for Updating Factors A, B										*/
/*    						(calls Nesterov_Matrix_Nnls function)								   	*/
/*    																							   	*/
/*                                                                           						*/
/* A. P. Liavas																						*/
/* Georgios Lourakis																				*/ 
/* Georgios Kostoulas																				*/
/* 15/11/2017                                              											*/
/*--------------------------------------------------------------------------------------------------*/
#include "ntf_functions.h"

using namespace Eigen;

void Workerjob(int my_rank, int n_J, int n_K, Ref<MatrixXd> A, const Ref<const MatrixXd> C_T_C, const Ref<const MatrixXd> A_T_A, 
				const Ref<const	MatrixXd> X_A, const Ref<const MatrixXd> Kr, int *Count, int *dis, double delta_1, double delta_2, 
				int factor, MPI_Comm comm){
	int n_I = A.rows();
	int R = A.cols();
	int I_local = Count[my_rank]/R;
	int i;
	MatrixXd Z(R, R);
	MatrixXd A_init(I_local, R);
	MatrixXd A_init_T(R, I_local);
	MatrixXd W_local(I_local, R);
	MatrixXd W_local_T(R, I_local);
	MatrixXd W = MatrixXd::Zero(n_I, R);
	MatrixXd W_T(R, n_I);
	MatrixXd A_T(R, n_I);
	MatrixXd temp;

	Z.noalias() = (C_T_C).cwiseProduct(A_T_A);								// Z_A = C^T*C .* B^T*B  or  Z_B = C^T*C .* A^T*A
	
	if (factor==2){
		for (i=0; i<n_K; i++){
			temp = X_A.block(i*n_I, 0, n_I, n_J);
			W.noalias() += temp * Kr.block(i*n_J, 0, n_J, R);				// W = X_B * Kr_B
		}
	}
	else
		W.noalias() = X_A * Kr;												// W = X_A * Kr_A
	
	W_T = W.transpose();
	MPI_Reduce_scatter(W_T.data(), W_local_T.data(), Count, MPI_DOUBLE, MPI_SUM, comm);						// Communication through layer	
	W_local = -W_local_T.transpose();
	
	A_init = A.block(dis[my_rank]/R, 0, I_local, R);
/////////////
		if (I_local!=0){
			Nesterov_Matrix_Nnls(Z, W_local, A_init, delta_1, delta_2);
			}
////////////
	//Nesterov_Matrix_Nnls(Z, W_local, A_init, delta_1, delta_2);

	A_init_T = A_init.transpose();
	MPI_Allgatherv(A_init_T.data(), Count[my_rank], MPI_DOUBLE, A_T.data(), Count, dis, MPI_DOUBLE, comm);	// Communication through layer
	A = A_T.transpose();
}
