/*--------------------------------------------------------------------------------------------------*/
/* 							Function for Updating Factors C											*/
/*    						(calls Nesterov_Matrix_Nnls function)								   	*/
/*    																							   	*/
/*                                                                           						*/
/* A. P. Liavas																						*/
/* Georgios Lourakis																				*/ 
/* Georgios Kostoulas																				*/
/* 15/11/2017                                              											*/
/*--------------------------------------------------------------------------------------------------*/
#include "ntf_functions.h"

using namespace Eigen;

void Workerjob_C(int my_rank, Ref<MatrixXd> C, Ref<MatrixXd> W_C, const Ref<const MatrixXd> B_T_B, const Ref<const MatrixXd> A_T_A,
  				const Ref<const	MatrixXd> X_C, const Ref<const MatrixXd> Kr, int *Count, int *dis, double delta_1, double delta_2,
  				 MPI_Comm comm){
	int n_K = C.rows();
	int R = C.cols();
	int I_local = Count[my_rank]/R;
	MatrixXd Z_C(R, R);
	MatrixXd C_init(I_local, R);
	MatrixXd C_init_T(R, I_local);
	MatrixXd W_local(I_local, R);
	MatrixXd W_local_T(R, I_local);
	MatrixXd W_C_T(R, n_K);
	MatrixXd C_T(R, n_K);

	Z_C.noalias() = (B_T_B).cwiseProduct(A_T_A);												// Z_C = B^T*B .* A^T*A
	
	W_C.noalias() = X_C * Kr;																	// W_C = X_C * Kr_C

	W_C_T = W_C.transpose();
	MPI_Reduce_scatter(W_C_T.data(), W_local_T.data(), Count, MPI_DOUBLE, MPI_SUM, comm);				// Communication through layer
	W_local = -W_local_T.transpose();

	C_init = C.block(dis[my_rank]/R, 0, I_local, R);
	Nesterov_Matrix_Nnls(Z_C, W_local, C_init, delta_1, delta_2);
	
	C_init_T = C_init.transpose();
	MPI_Allgatherv(C_init_T.data(), Count[my_rank], MPI_DOUBLE, C_T.data(), Count, dis, MPI_DOUBLE, comm);	// Communication through layer
	C = C_T.transpose();
}
